class RentForm < ApplicationForm
  include DatesNowValidator

  FIELDS = %i[
    address
    latitude
    longitude
    location_address
    name
    from
    to
    guests
    kids
    boat_types
    is_skippered
    is_my_price
    my_price_currency
    my_price
    watersports
    need_transfer
    comments
    email
    name
    phone_number
    password
    sms_code
    staying_overnight
    uploaded_license_file
    hours_per_day
  ].freeze

  attr_accessor *FIELDS

  validates :address,
            :latitude,
            :longitude,
            :from,
            :to,
            :email,
            :name,
            :phone_number, presence: true

  validates :phone_number, phony_plausible: true
  validates :email, email: { strict_mode: true }

  validates :kids,   numericality: { greater_than_or_equal_to: 0 }, if: -> { kids.present? }
  validates :guests, numericality: { greater_than_or_equal_to: 0 }

  validates :my_price,          numericality: { greater_than_or_equal_to: 0 },                           if: -> { !my_price.to_i.zero? }
  validates :my_price_currency, presence: true, length: { minimum: 3, maximum: 3 }, allow_blank: false, if: -> { !my_price.to_i.zero? }

  # validates :uploaded_license_file, presence: true, allow_blank: false, if: -> { is_skippered == 'false' }
  validate :validate_license

  validate :validate_dates
  validate :validate_boat_types
  validate :validate_address

  attr_reader :is_need_bareboat_license

  def initialize(params = nil, is_need_bareboat_license = true)
    self.send(:is_my_price=, !params[:my_price].to_i.zero?) unless params.nil?
    @is_need_bareboat_license = is_need_bareboat_license

    super params
  end

  def validate_license
    return unless @is_need_bareboat_license
    return unless is_skippered.eql?('false')

    unless uploaded_license_file.present?
      errors.add(:uploaded_license_file, :presence)
    end
  end

  def button_name
    I18n.t(:new_rent_order)
  end

  def validate_dates
    check_dates errors, from, to
  end

  def validate_boat_types
    if boat_types.nil? || boat_types.empty?
      errors.add(:boat_types, :absent)
    else
      unless BoatType.where(id: boat_types.map(&:to_i)).exists?
        errors.add(:boat_types, :absent)
      end
    end
  end

  def validate_address
    conditions = [
      address.empty?,
      latitude.to_f.zero?,
      longitude.to_f.zero?,
      latitude.to_i < -90,
      latitude.to_i > 90,
      longitude.to_i < -180,
      longitude.to_i > 180,
    ]

    if conditions.any?
      errors.add(:address, :invalid_address)
    end
  end

end
