module Schemas
  class BaseBoatSchema < ::Schemas::Base
    def schema
      result = {
          properties: {
              id: {
                  type: %i[null number],
                  cast: ->(value) { normalize_integer(value) }
              },
              name: {
                  type: [:null, :string],
                  cast: ->(value) { clear_html(strip_string(value)) },
                  rule: ->(parents) { rule_name(parents) }
              },
              boat_type_id: {
                  enum: [nil, -1] + Dicts::BoatType::ALL.collect(&:id),
                  cast: ->(value) { normalize_integer(value) },
                  rule: ->(parents) { rule_boat_type_id(parents) }
              },
              boat_model: {
                  type: %i[null string],
                  cast: ->(value) { clear_html(strip_string(value)) }
              },
              builder: {
                  type: %i[null string],
                  cast: ->(value) { clear_html(strip_string(value)) }
              },
              length: {
                  type: :hash,
                  required: %i[value length_id],
                  properties: {
                      value: {
                          type: %i[null number string],
                          cast: ->(value) { to_float(value) }
                      },
                      length_id: {
                          enum: Dicts::Length.all.collect(&:id),
                          cast: ->(value) { normalize_integer(value) }
                      }
                  }
              },
              crew_total: {
                  type: %i[null number],
                  cast: ->(value) { normalize_integer(value) }
              },
              guest_cabins: {
                  type: %i[null number],
                  cast: ->(value) { normalize_integer(value) }
              },
              guests_total: {
                  type: %i[null number],
                  cast: ->(value) { normalize_integer(value) }
              },
              maximum_guests_during_cruise: {
                  type: %i[null number],
                  cast: ->(value) { normalize_integer(value) },
                  rule: ->(parents) { rule_maximum_guests_during_cruise(parents) }
              },
              boat_locations_attributes: {
                  type: %i[null array],
                  items: {
                      type: :hash,
                      required: %i[latitude longitude address is_main _destroy],
                      cast: ->(value) do
                        value[:latitude]  = to_big_decimal value[:latitude]
                        value[:longitude] = to_big_decimal value[:longitude]
                        value[:address]   = strip_string   value[:address]
                        value[:is_main]   = to_bool        value[:is_main]
                        value[:_destroy]  = to_bool        value[:_destroy]

                        value[:latitude].nil? || value[:longitude].nil? || !value[:address].present? ? nil : value
                      end
                  },
                  cast: ->(value) { forced_array(value) }
              }
          }
      }

      result[:properties][:rent_prices] = {
          type: [:null, :array],                                                                                      # это первая версия, где я пробую разобраться с массивом объектов (версия покруче тут - app/getboat/schemas/base_boat_schema.rb - с помощью :items)
          cast: ->(value) do
            value&.inject([]) do |res, el|
              el[:_delete]  = to_bool el[:_delete]
              el[:currency] = normalize_integer el[:currency]
              el[:discount] = to_float el[:discount]
              el[:duration] = to_float el[:duration]
              el[:season]   = normalize_integer el[:season]
              el[:uom]      = normalize_integer el[:uom]
              el[:value]    = normalize_integer(el[:value].is_a?(String) ? el[:value].split(/[, ]/).join : el[:value])
              res << el
              res
            end
          end
      }

      result[:properties][:sale_price] = {
          type: [:null, :hash],
          properties: {
              currency_id: {
                  enum: Dicts::Currency.all.collect(&:id),
                  cast: ->(value) { normalize_integer(value) }
              },
              value: {
                  type: %i[number null string],
                  cast: ->(value) do
                    val = value.is_a?(String) ? value.split(/[, ]/).join : value
                    normalize_integer val
                  end
              },
              discount: {
                  type: %i[number null string],
                  cast: ->(value) { to_float(value) }
              },
          }
      }

      result[:properties][:boat_photos_attributes] = {
          type: [:null, :hash],
          cast: ->(value) { nullify_empty(value) },
          rule: ->(parents) { rule_boat_photos_attributes(parents)}
      }

      result
    end


    def rule_name(parents)
      messages = {
          present:        I18n.t('errors.boat.name.blank'),
          length_between: I18n.t('errors.boat.name.length_invalid', min: 2, max: 255)
      }
      rule(:name, parents, messages) { present? & length_between?(2..255) }
    end


    def rule_boat_type_id(parents)
      messages = {
          present: I18n.t('errors.boat.boat_type_id.blank'),
          gt:      I18n.t('errors.boat.boat_type_id.blank')
      }
      rule(:boat_type_id, parents, messages) { present? & gt?(0) }
    end


    def rule_maximum_guests_during_cruise(parents)
      messages = {
          present: I18n.t('errors.boat.maximum_guests_during_cruise.blank'),
          gt:      I18n.t('errors.boat.maximum_guests_during_cruise.blank')
      }
      rule(:maximum_guests_during_cruise, parents, messages) { present? & gt?(0) }
    end


    def rule_boat_photos_attributes(parents)
      messages = { present: I18n.t('errors.boat.boat_photos_attributes.blank') }
      rule(:boat_photos_attributes, parents, messages) { present? }
    end


    def validate_location
      unless boat_locations_attributes
        errors.add :boat_locations, I18n.t('errors.boat.boat_locations.blank')
        return
      end

      bls  = boat_locations_attributes.compact.select { |loc| !loc[:_destroy] }
      mbls = bls.select { |loc| loc[:is_main] }

      if bls.size.zero?
        errors.add :boat_locations, I18n.t('errors.boat.boat_locations.blank')
        return
      end

      if bls.size > 5
        errors.add :boat_locations, I18n.t('errors.boat.boat_locations.limit', max: 5)
        return
      end

      if mbls.size == 0
        errors.add :boat_locations, I18n.t('errors.boat.boat_locations.is_main')
        return
      end
    end


    def validate_length
      errors.add(:length_value, I18n.t('errors.boat.length.blank')) unless length[:value]&.nonzero?
    end


    def validate_crew_total
      errors.add(:crew_total, I18n.t('errors.boat.crew_total.blank')) unless crew_total.present?
    end


    def validate_guests_total
      errors.add(:guests_total, I18n.t('errors.boat.guests_total.blank')) unless guests_total.present?
    end


    def validate_guest_cabins
      errors.add(:guest_cabins, I18n.t('errors.boat.guest_cabins.blank')) unless guest_cabins.present?
    end


    def validate_prices_and_rent_prices
      rent_prices_present = rent_prices.present? && rent_prices.count { |price| price[:_delete] } < rent_prices.count
      sale_price_present  = sale_price.present? && sale_price[:value]&.>(0)
      return if rent_prices_present && sale_price_present

      if !(rent_prices_present || sale_price_present)
        errors.add :rent_prices, I18n.t('errors.boat.prices.any')
        errors.add :sale_price, I18n.t('errors.boat.prices.any')
      end
    end


    def validate_rent_prices
      return unless rent_prices.present?

      rent_prices.each do |price|
        if price[:value].nil? || price[:value].zero?
          errors.add :rent_prices, I18n.t('errors.boat.rent_prices.invalid')
          break
        end
        if price[:duration].nil? || price[:duration].zero?
          errors.add :rent_prices, I18n.t('errors.boat.rent_prices.invalid_duration')
          break
        end
      end
    end
  end
end