module Schemas
  module Site
    module Lease
      class CreatePersonalInquirySchema < ::Schemas::Base
        def schema
          result = {
              required: %i[
                  id
                  from
                  until
                  guests
                  kids
                  is_skippered
                  comments
                  email
                  name
                  _known_is_skippered
                  _known_maximum_guests_during_cruise
              ],
              properties: {
                  id: {
                      type: %i[null number],
                      cast: ->(value) { normalize_integer(value) }
                  },
                  from: {
                      type: %i[null date],
                      cast: ->(value) { to_date(value) }
                  },
                  until: {
                      type: %i[null date],
                      cast: ->(value) { to_date(value) }
                  },
                  guests: {
                      type: %i[null number],
                      cast: ->(value) { normalize_integer(value) },
                      rule: ->(parents) { rule_guests(parents) }
                  },
                  kids: {
                      type: %i[null number],
                      cast: ->(value) { normalize_integer(value) },
                      rule: ->(parents) { rule_kids(parents) }
                  },
                  is_skippered: {
                      type: :boolean,
                      cast: ->(value) { to_bool(value) }
                  },
                  watersports: {
                      type: :boolean,
                      cast: ->(value) { to_bool(value) }
                  },
                  transfer: {
                      type: :boolean,
                      cast: ->(value) { to_bool(value) }
                  },
                  comments: {
                      type: :string,
                      cast: ->(value) { clear_html(value) },
                      rule: ->(parents) { rule_comments(parents) }
                  },
                  email: {
                      type: %i[null string],
                      cast: ->(value) { strip_string(value) },
                      rule: ->(parents) { rule_email(parents) }
                  },
                  name: {
                      type: %i[null string],
                      cast: ->(value) { strip_string(value) },
                      rule: ->(parents) { rule_name(parents) }
                  },
                  phone: {
                      type: %i[null string],
                      cast: ->(value) { strip_string(value) },
                      rule: ->(parents) { rule_phone(parents) }
                  },
                  password: {
                      type: %i[null string]
                  },
                  _known_is_skippered: {
                      type: %i[string],
                      cast: ->(value) { strip_string(value) }
                  },
                  _known_maximum_guests_during_cruise: {
                      type: :number,
                      cast: ->(value) { normalize_integer(value) }
                  }
              }
          }

          result
        end

        def validate_additional
          validate_dates
          validate_is_skippered
          validate_guests_total
          validate_kids_guests
        end

        def validate_dates
          unless from
            errors.add :from, I18n.t('errors.rent_form.from_until')
            return
          end

          unless send(:until)
            errors.add :until, I18n.t('errors.rent_form.from_until')
            return
          end
        end

        def validate_is_skippered
          return if _known_is_skippered == 'null' # если 'null' - значит не указано, значит можно всё
          return if to_bool(_known_is_skippered) == is_skippered

          errors.add :is_skippered, 'It is impossible'
        end

        # если юзер указал детей, больше и равно чем guests - сообщим ему об этом, заявку не создаём
        def validate_kids_guests
          return if kids.nil?
          return if guests > kids
          return if errors.messages[:guests].present?

          errors.add :kids, I18n.t('errors.messages.too_much_and', max: guests, and: I18n.t('activemodel.attributes.rent_form.guests'))
        end

        # если слишком много людей хотят вместить на лодку
        def validate_guests_total
          return if guests.nil?
          return if _known_maximum_guests_during_cruise >= guests

          errors.add :guests, I18n.t('errors.messages.too_much_eq_and', max: _known_maximum_guests_during_cruise, and: I18n.t('attributes.maximum_capacity_tip'))
        end

        def rule_guests(parents)
          messages = {
              present: I18n.t('errors.messages.too_low', min: 0),
              gt:      I18n.t('errors.messages.too_low', min: 0)
          }
          rule(:guests, parents, messages) { present? & gt?(0) }
        end

        def rule_kids(parents)
          return if kids.nil?

          messages = {
              gt_eq:      I18n.t('errors.messages.too_low_eq', min: 0)
          }
          rule(:kids, parents, messages) { gt_eq?(0) }
        end

        def rule_comments(parents)
          return unless comments.present?

          messages = {
              length_between: I18n.t('errors.messages.length_invalid', min: 5, max: 2048)
          }
          rule(:comments, parents, messages) { length_between?(5..2048) }
        end

        def rule_name(parents)
          messages = {
              present:     I18n.t('errors.messages.invalid'),
              max_length:  I18n.t('errors.messages.length_invalid', min: ::ValidationConstraints::USER_NAME_MIN_LENGTH, max: ::ValidationConstraints::USER_NAME_MAX_LENGTH)
          }
          rule(:name, parents, messages) { present? & max_length?(::ValidationConstraints::USER_NAME_MAX_LENGTH) }
        end

        def rule_phone(parents)
            messages = {
                present: I18n.t('errors.messages.invalid')
            }
            rule(:phone, parents, messages) { present? }
        end

        def rule_email(parents)
          messages = {
              present:     I18n.t('errors.messages.invalid'),
              max_length:  I18n.t('errors.messages.length_invalid', min:0, max:100),
              email_mx:    I18n.t('errors.messages.invalid')
          }
          rule(:email, parents, messages) { present? & max_length?(100) & format?(ValidationConstraints::EMAIL_REGEX) }
        end
      end
    end
  end
end