module Lease
  class BidSerializer < ::AbstractSerializer

    extend ::Site::TimeHelper
    extend ::Custom::CurrencyHelper
    extend ::ActionView::Helpers::NumberHelper

    class << self

      def available_attributes
        %i[
          id
          account_id
          inquiry_id
          boat_id
          boat
          price
          price_percent
          cancellation_type
          human_payment_prices
          currency
          state
          from
          to
          period
          duration
          human_price
          human_price_vat_incl
          comments
          extra
        ].freeze
      end

      def id(bid)
        { id: bid.id }
      end

      def account_id(bid)
        { account_id: bid.account_id }
      end

      def inquiry_id(bid)
        { inquiry_id: bid.inquiry_id }
      end

      def boat_id(bid)
        { boat_id: bid.boat_id }
      end

      def boat(bid)
        boat            = bid.boat
        boat_serialized = BoatSerializer.serialize boat, attributes: %i[
          name
          boat_length
          guests_total
          guest_cabins
          crew_total
          boat_stars
          builder
          built_year
        ]
        # backburner fails with "Exception ActiveJob::SerializationError -> Unsupported argument type: BigDecimal"
        boat_serialized[:boat_length][:value] = boat_serialized[:boat_length][:value].to_i
        {
          boat: {
            img_url:         (boat.boat_photo.picture.url(:thumb) if boat.boat_photo.present?),
            gallery_img_url: (boat.boat_photo.picture.url(:gallery) if boat.boat_photo.present?),
            type:            (boat.boat_type.name rescue nil),
            **boat_serialized
          }
        }
      end

      def price(bid)
        { price: bid.price }
      end

      def price_percent(bid)
        { price_percent: bid.price_percent }
      end

      def cancellation_type(bid)
        return { cancellation_type: nil } if bid.cancellation_type.nil?

        ct = ::Dicts::Lease::OfferCancelation.find bid.cancellation_type

        { cancellation_type: {
                id:        ct.id,
                title:     I18n.t('lease.offer_cancelation.%s.title'     % ct.index),
                short:     I18n.t('lease.offer_cancelation.%s.short'     % ct.index),
                full_html: I18n.t('lease.offer_cancelation.%s.full_html' % ct.index)
            }
        }
      end

      def human_payment_prices(bid)
        if bid.price.present? && bid.price_percent.present?
          first_price = bid.price * bid.price_percent / 100
          {
              human_payment_prices: {
                  first_payment: num_to_cur(first_price, bid.currency, @opts[:locale]),
                  second_payment: num_to_cur((bid.price - first_price), bid.currency, @opts[:locale])
              }
          }
        end
      end

      def currency(bid)
        { currency: bid.currency }
      end

      def human_price(bid)
        val = nil
        if bid.price.present? && bid.currency.present?
          val = num_to_cur(bid.price, bid.currency, @opts[:locale])
        end
        { human_price: val }
      end

      def human_price_vat_incl(bid)
        val = nil
        if bid.price.present? && bid.currency.present?
          val = bid.price / 100.0 * (100 + bid.vat)
          val = num_to_cur(val.to_i, bid.currency, @opts[:locale])
        end

        { human_price_vat_incl: val }
      end

      def period(bid)
        { period: '%s - %s' % [from(bid)[:from], to(bid)[:to]] }
      end

      def duration(bid)
        distance = (bid.to - bid.from).to_i / 24 / 60 / 60
        distance = distance.zero? ? 1 : distance
        uom      = I18n.t('activerecord.labels.lease/bid.day', count: distance)
        {
          duration: {
            distance: distance,
            uom:      uom
          }
        }
      end

      def from(bid)
        { from: format_date(bid.from, @opts[:locale]) }
      end

      def to(bid)
        { to: format_date(bid.to, @opts[:locale]) }
      end

      def state(bid)
        { state: bid.state }
      end

      def comments(bid)
        { comments: bid.comments }
      end

      def extra(bid)
        extra = %i[
          extra_captain
          extra_towels
          extra_hostess
          extra_cleaning
          extra_chef
          extra_fees
          extra_apa
        ].map do |attr|
          [attr, bid.send(attr)]
        end.to_h.select do |_, value|
          !value.nil?
        end.map do |a,v|
          if a == :extra_apa
            [a, '%s %' % v ]
          else
            [a, num_to_cur(v,bid.currency, @opts[:locale])]
          end
        end.to_h

        { extra: extra }
      end
    end
  end
end
