module Lease
  class InquirySerializer < ::AbstractSerializer

    extend ::Site::TimeHelper

    class << self

      def available_attributes
        %i[
          account_id
          address
          boat_types
          comments
          created_at
          created_at_full
          duration
          email
          from
          guests
          hours_per_day
          id
          is_my_price
          is_skippered
          kids
          latitude
          longitude
          my_price
          my_price_currency
          name
          need_transfer
          period
          period_formatted
          phone_number
          responsible_managers
          state
          to
          token
          watersports
        ].freeze
      end

      def id(inquiry)
        { id: inquiry.id }
      end

      def account_id(inquiry)
        { account_id: inquiry.account_id }
      end

      def state(inquiry)
        { state: inquiry.state }
      end

      def token(inquiry)
        { token: inquiry.token }
      end

      def is_skippered(inquiry)
        { is_skippered: inquiry.is_skippered }
      end

      def guests(inquiry)
        { guests: inquiry.guests }
      end

      def kids(inquiry)
        { kids: inquiry.kids }
      end

      def watersports(inquiry)
        { watersports: inquiry.watersports }
      end

      def need_transfer(inquiry)
        { need_transfer: inquiry.need_transfer }
      end

      def comments(inquiry)
        { comments: inquiry.comments }
      end

      def created_at(inquiry)
        { created_at: format_date(inquiry.created_at, @opts[:locale]) }
      end

      def created_at_full(inquiry)
        { created_at_full: format_date(inquiry.created_at, @opts[:locale], true) }
      end

      def address(inquiry)
        { address: inquiry.address }
      end

      def latitude(inquiry)
        { latitude: inquiry.latitude.to_f }
      end

      def longitude(inquiry)
        { longitude: inquiry.longitude.to_f }
      end

      def email(inquiry)
        { email: inquiry.client_account.account.user.email }
      end

      def name(inquiry)
        { name: inquiry.name }
      end

      def phone_number(inquiry)
        { phone_number: inquiry.phone_number }
      end

      def period(inquiry)
        { period: '%s - %s' % [from(inquiry)[:from], to(inquiry)[:to]] }
      end

      def period_formatted(inquiry)
        frm = format_date(inquiry.from, @opts[:locale], false, true)
        tto = format_date(inquiry.to, @opts[:locale], false, true)

        { period: {
            from: frm,
            to:   tto
          }
        }
      end

      def duration(inquiry)
        distance = (inquiry.to - inquiry.from).to_i / 24 / 60 / 60
        distance = distance.zero? ? 1 : distance + 1
        uom      = I18n.t('activerecord.labels.lease/bid.day', count: distance)
        {
          duration: {
            distance: distance,
            uom:      uom
          }
        }
      end

      def from(inquiry)
        { from: format_date(inquiry.from, @opts[:locale]) }
      end

      def to(inquiry)
        { to: format_date(inquiry.to, @opts[:locale]) }
      end

      def is_my_price(inquiry)
        { is_my_price: inquiry.is_my_price }
      end

      def my_price(inquiry)
        { my_price: inquiry.my_price }
      end

      def my_price_currency(inquiry)
        { my_price_currency: inquiry.my_price_currency }
      end

      def boat_types(inquiry)
        boat_types = BoatType.where(id: inquiry.boat_type_ids).includes(:translations)
        {
          boat_types: boat_types.map do |bt|
            {
              id:   bt.id,
              name: bt.name,
              icon: bt.icon.url
            }
          end
        }
      end

      def responsible_managers(inquiry)
        res = inquiry.responsible_managers.includes(:user).map do |acc|
          { id:    acc.user.id,
            email: acc.user.email
          }
        end
        { responsible_managers: res }
      end

      def hours_per_day(inquiry)
        { hours_per_day: inquiry.hours_per_day }
      end

    end
  end
end
